@@ -6,6 +6,7 @@ from typing import Any
 from typing import Callable
 from typing import cast
 from typing import List
+from typing import Literal
 from typing import Mapping
 from typing import Optional
 from typing import TextIO
@@ -25,7 +26,7 @@ from inifix.validation import validate_inifile_schema
 
 __all__ = ["load", "dump"]
 
-SECTION_REGEXP = re.compile(r"\[.+\]\s*")
+SECTION_REGEXP = re.compile(r"\[(?P<title>[^(){}\[\]]+)\]\s*")
 
 
 def bool_caster(s: str) -> bool:
@@ -46,7 +47,6 @@ def str_caster(s: str) -> str:
 
 CASTERS: List[Callable] = [
     int,
-    ENotationIO.decode,
     float,
     bool_caster,
     str_caster,
@@ -106,12 +106,46 @@ def _normalize_data(data: str) -> List[str]:
     return lines
 
 
+def _next_token(data: str, pattern: str, start: Literal[0, 1]) -> Tuple[str, int]:
+    pos: int = start
+    while pos < len(data) and not re.match(pattern, data[pos]):
+        pos += 1
+    if start == 1:
+        end = pos + 1
+    else:
+        end = pos
+    token = data[:end]
+    return token, pos
+
+
+def _split_tokens(data: str) -> List[str]:
+    tokens = []
+    pattern = r"\s"
+    start: Literal[0, 1] = 0
+    data = data.strip()
+    while True:
+        token, pos = _next_token(data, pattern, start)
+        tokens.append(token)
+        data = data[pos + 1 :].strip()
+        if not data:
+            break
+        if data[0] in ('"', "'"):
+            pattern = data[0]
+            start = 1
+        else:
+            start = 0
+    return tokens
+
+
 def _tokenize_line(
-    line: str, file: TextIO, line_number: int
+    line: str, line_number: int, file: Optional[TextIO]
 ) -> Tuple[str, List[Scalar]]:
-    key, *raw_values = line.split()
+    key, *raw_values = _split_tokens(line)
     if not raw_values:
-        raise ValueError(f"Failed to parse {file}:{line_number}:\n{line}")
+        if file is None:
+            raise ValueError(f"Failed to parse line {line_number}: {line!r}")
+        else:
+            raise ValueError(f"Failed to parse {file}:{line_number}:\n{line}")
 
     values = []
     for val in raw_values:
@@ -130,24 +164,19 @@ def _tokenize_line(
     return key, values
 
 
-def _from_file_descriptor(file: TextIO) -> InifixConfT:
-    data = file.read()
-
+def _from_string(data: str, file: Optional[TextIO] = None) -> InifixConfT:
     # see https://github.com/python/mypy/issues/6463
     container: InifixConfT = {}  # type: ignore[assignment]
     lines = _normalize_data(data)
-    if not "".join(lines):
-        raise ValueError(f"{file.name!r} appears to be empty.")
-
     section = Section()  # the default target is a nameless section
     for line_number, line in enumerate(lines, start=1):
         if not line:
             continue
-        match = re.match(SECTION_REGEXP, line)
+        match = re.fullmatch(SECTION_REGEXP, line)
         if match is not None:
             if section:
                 section._dump_to(container)
-            section = Section(name=match.group().strip("[]"))
+            section = Section(name=match["title"])
             continue
 
         values: Union[Scalar, List[Scalar]]
@@ -159,6 +188,14 @@ def _from_file_descriptor(file: TextIO) -> InifixConfT:
     return container
 
 
+def _from_file_descriptor(file: TextIO) -> InifixConfT:
+    data = file.read()
+    lines = _normalize_data(data)
+    if not "".join(lines):
+        raise ValueError(f"{file.name!r} appears to be empty.")
+    return _from_string(data, file=file)
+
+
 def _from_path(file: PathLike) -> InifixConfT:
     file = os.fspath(file)
     with open(file) as fh:
@@ -169,7 +206,7 @@ def _from_path(file: PathLike) -> InifixConfT:
 
 
 def _encode(v: Scalar) -> str:
-    if isinstance(v, (float, int)):
+    if isinstance(v, float):
         return ENotationIO.encode_preferential(v)
     return str(v)
 
@@ -219,6 +256,10 @@ def load(source: Union[InifixConfT, PathLike, TextIO], /) -> InifixConfT:
     return source
 
 
+def loads(source: str, /) -> InifixConfT:
+    return _from_string(source)
+
+
 def dump(data: InifixConfT, /, file: Union[PathLike, TextIOBase]) -> None:
     """
     Write data to a file.
@@ -238,3 +279,11 @@ def dump(data: InifixConfT, /, file: Union[PathLike, TextIOBase]) -> None:
         _write_to_buffer(data, file)  # type: ignore
     except AttributeError:
         _write_to_file(data, file)
+
+
+def dumps(data: InifixConfT, /) -> str:
+    from io import StringIO
+
+    s = StringIO()
+    dump(data, file=s)
+    return s.getvalue()
